@@ -35,7 +35,7 @@ would appreciate credit if this program or parts of it are used.
 
 #include <ctype.h>
 
-#include "tcl.h"
+#include "tclInt.h"
 #include "string.h"
 
 #include "exp_tty_in.h"
@@ -43,6 +43,12 @@ would appreciate credit if this program or parts of it are used.
 #include "exp_prog.h"
 #include "exp_command.h"
 #include "exp_log.h"
+#include "exp_event.h" /* exp_get_next_event decl */
+
+/* Tcl 8.5+ moved this internal - needed for when I compile expect against 8.5. */
+#ifndef TCL_REG_BOSONLY
+#define TCL_REG_BOSONLY 002000
+#endif
 
 typedef struct ThreadSpecificData {
     Tcl_Obj *cmdObjReturn;
@@ -124,9 +130,9 @@ struct input {
  */
 
 struct input *
-expStateToInput(hash,esPtr)
-    ExpState *esPtr;
-    Tcl_HashTable *hash;
+expStateToInput(
+    Tcl_HashTable *hash,
+    ExpState *esPtr)
 {
     Tcl_HashEntry *entry = Tcl_FindHashEntry(hash,(char *)esPtr);
 
@@ -138,10 +144,10 @@ expStateToInput(hash,esPtr)
 }
 
 void
-expCreateStateToInput(hash,esPtr,inp)
-    ExpState *esPtr;
-    Tcl_HashTable *hash;
-    struct input *inp;
+expCreateStateToInput(
+    Tcl_HashTable *hash,
+    ExpState *esPtr,
+    struct input *inp)
 {
     Tcl_HashEntry *entry;
     int newPtr;
@@ -150,12 +156,15 @@ expCreateStateToInput(hash,esPtr,inp)
     Tcl_SetHashValue(entry,(ClientData)inp);
 }
 
-static void free_input();
-static void free_keymap();
-static void free_output();
-static void free_action();
-static struct action *new_action();
-static int inter_eval();
+static void free_input(Tcl_Interp *interp, struct input *i);
+static void free_keymap(struct keymap *km);
+static void free_output(Tcl_Interp *interp, struct output *o);
+static void free_action(struct action *a);
+static struct action *new_action(struct action **base);
+static int inter_eval(
+    Tcl_Interp *interp,
+    struct action *action,
+    ExpState *esPtr);
 
 /* intMatch() accepts user keystrokes and returns one of MATCH,
 CANMATCH, or CANTMATCH.  These describe whether the keystrokes match a
@@ -186,27 +195,28 @@ we're ready).  The other is to return can-match.
 */
 
 static int
-intMatch(esPtr,keymap,km_match,matchLen,skip,info)
-    ExpState *esPtr;
-    struct keymap *keymap;	/* linked list of keymaps */
-    struct keymap **km_match;	/* keymap that matches or can match */
-    int *matchLen;		/* # of bytes that matched */
-    int *skip;			/* # of chars to skip */
-    Tcl_RegExpInfo *info;
+intMatch(
+    ExpState *esPtr,
+    struct keymap *keymap,	/* linked list of keymaps */
+    struct keymap **km_match,	/* keymap that matches or can match */
+    int *matchLen,		/* # of bytes that matched */
+    int *skip,			/* # of chars to skip */
+    Tcl_RegExpInfo *info)
 {
-    char *string;
+    Tcl_UniChar *string;
     struct keymap *km;
     char *ks;		/* string from a keymap */
 
-    char *start_search;	/* where in string to start searching */
+    Tcl_UniChar *start_search;	/* where in string to start searching */
     int offset;		/* # of chars from string to start searching */
 
-    char *string_end;
-    int stringBytes, bytesThisChar;
+    Tcl_UniChar *string_end;
+    int numchars;
     int rm_nulls;		/* skip nulls if true */
     Tcl_UniChar ch;
 
-    string = Tcl_GetStringFromObj(esPtr->buffer,&stringBytes);
+    string   = esPtr->input.buffer;
+    numchars = esPtr->input.use; /* Actually #chars */
 
     /* assert (*km == 0) */
 
@@ -214,13 +224,13 @@ intMatch(esPtr,keymap,km_match,matchLen,skip,info)
     /* is lengthy and has no key maps.  Otherwise it would mindlessly */
     /* iterate on each character anyway. */
     if (!keymap) {
-	*skip = stringBytes;
+	*skip = numchars;
 	return(EXP_CANTMATCH);
     }
 
     rm_nulls = esPtr->rm_nulls;
 
-    string_end = string + stringBytes;
+    string_end = string + numchars;
 
     /*
      * Maintain both a character index and a string pointer so we
@@ -229,16 +239,16 @@ intMatch(esPtr,keymap,km_match,matchLen,skip,info)
 
     for (start_search = string, offset = 0;
 	 start_search < string_end;
-	 start_search += bytesThisChar, offset++) {
+	 start_search ++, offset++) {
 
-	bytesThisChar = Tcl_UtfToUniChar(start_search, &ch);
+	ch = *start_search;
 	
 	if (*km_match) break; /* if we've already found a CANMATCH */
 			/* don't bother starting search from positions */
 			/* further along the string */
 
 	for (km=keymap;km;km=km->next) {
-	    char *s;	/* current character being examined */
+	    Tcl_UniChar *s;	/* current character being examined */
 
 	    if (km->null) {
 		if (ch == 0) {
@@ -248,13 +258,13 @@ intMatch(esPtr,keymap,km_match,matchLen,skip,info)
 		    return(EXP_MATCH);
 	        }
 	    } else if (!km->re) {
-		int slen, kslen;
+		int kslen;
 		Tcl_UniChar sch, ksch;
 		
 		/* fixed string */
 
 		ks = Tcl_GetString(km->keys);
-		for (s = start_search;; s += slen, ks += kslen) {
+		for (s = start_search;; s++, ks += kslen) {
 		    /* if we hit the end of this map, must've matched! */
 		    if (*ks == 0) {
 			*skip = start_search-string;
@@ -274,7 +284,7 @@ intMatch(esPtr,keymap,km_match,matchLen,skip,info)
 			break;
 		    }
 
-		    slen = Tcl_UtfToUniChar(s, &sch);
+		    sch = *s;
 		    kslen = Tcl_UtfToUniChar(ks, &ksch);
 		    
 		    if (sch == ksch) continue;
@@ -289,18 +299,23 @@ intMatch(esPtr,keymap,km_match,matchLen,skip,info)
 		Tcl_RegExp re;
 		int flags;
 		int result;
+		Tcl_Obj* buf;
 
 		re = Tcl_GetRegExpFromObj(NULL, km->keys,
 			TCL_REG_ADVANCED|TCL_REG_BOSONLY|TCL_REG_CANMATCH);
 		flags = (offset > 0) ? TCL_REG_NOTBOL : 0;
 
-		result = Tcl_RegExpExecObj(NULL, re, esPtr->buffer, offset,
+		/* ZZZ: Future optimization: Avoid copying */
+		buf = Tcl_NewUnicodeObj (esPtr->input.buffer, esPtr->input.use);
+		Tcl_IncrRefCount (buf);
+		result = Tcl_RegExpExecObj(NULL, re, buf, offset,
 			-1 /* nmatches */, flags);
+		Tcl_DecrRefCount (buf);
 		if (result > 0) {
 		    *km_match = km;
 		    *skip = start_search-string;
 		    Tcl_RegExpGetInfo(re, info);
-		    *matchLen = Tcl_UtfAtIndex(start_search,info->matches[0].end) - start_search;
+		    *matchLen = info->matches[0].end;
 		    return EXP_MATCH;
 		} else if (result == 0) {
 		    Tcl_RegExpGetInfo(re, info);
@@ -337,15 +352,16 @@ intMatch(esPtr,keymap,km_match,matchLen,skip,info)
 
 /* put regexp result in variables */
 static void
-intRegExpMatchProcess(interp,esPtr,km,info,offset)
-     Tcl_Interp *interp;
-     ExpState *esPtr;
-     struct keymap *km;	/* ptr for above while parsing */
-     Tcl_RegExpInfo *info;
-     int offset;
+intRegExpMatchProcess(
+    Tcl_Interp *interp,
+    ExpState *esPtr,
+    struct keymap *km,	/* ptr for above while parsing */
+    Tcl_RegExpInfo *info,
+    int offset)
 {
     char name[20], value[20];
     int i;
+    Tcl_Obj* buf = Tcl_NewUnicodeObj (esPtr->input.buffer,esPtr->input.use);
 
     for (i=0;i<=info->nsubs;i++) {
 	int start, end;
@@ -369,22 +385,23 @@ intRegExpMatchProcess(interp,esPtr,km,info,offset)
 
 	/* string itself */
 	sprintf(name,"%d,string",i);
-	val = Tcl_GetRange(esPtr->buffer, start, end);
+	val = Tcl_GetRange(buf, start, end);
 	expDiagLog("interact: set %s(%s) \"",INTER_OUT,name);
 	expDiagLogU(expPrintifyObj(val));
 	expDiagLogU("\"\r\n");
 	Tcl_SetVar2Ex(interp,INTER_OUT,name,val,0);
     }
+    Tcl_DecrRefCount (buf);
 }
 
 /*
  * echo chars
  */ 
 static void
-intEcho(esPtr,skipBytes,matchBytes)
-    ExpState *esPtr;
-    int skipBytes;
-    int matchBytes;
+intEcho(
+    ExpState *esPtr,
+    int skipBytes,
+    int matchBytes)
 {
     int seenBytes;	/* either printed or echoed */
     int echoBytes;
@@ -400,8 +417,8 @@ intEcho(esPtr,skipBytes,matchBytes)
 	offsetBytes = seenBytes;
     }
 
-    (void) expWriteChars(esPtr,
-		   Tcl_GetString(esPtr->buffer) + offsetBytes,
+    (void) expWriteCharsUni(esPtr,
+			    esPtr->input.buffer + offsetBytes,
 		   echoBytes);
 
     esPtr->echoed = matchBytes + skipBytes - esPtr->printed;
@@ -412,22 +429,25 @@ intEcho(esPtr,skipBytes,matchBytes)
  * Returns # of bytes read or negative number (EXP_XXX) indicating unusual event.
  */
 static int
-intRead(interp,esPtr,warnOnBufferFull,interruptible,key)
-    Tcl_Interp *interp;
-    ExpState *esPtr;
-    int warnOnBufferFull;
-    int interruptible;
-    int key;
+intRead(
+    Tcl_Interp *interp,
+    ExpState *esPtr,
+    int warnOnBufferFull,
+    int interruptible,
+    int key)
 {
-    char *eobOld;  /* old end of buffer */
+    Tcl_UniChar *eobOld;  /* old end of buffer */
     int cc;
-    int size;
-    char *str;
+    int numchars;
+    Tcl_UniChar *str;
 
-    str = Tcl_GetStringFromObj(esPtr->buffer,&size);
-    eobOld = str+size;
+    str      = esPtr->input.buffer;
+    numchars = esPtr->input.use;
+    eobOld   = str + numchars;
 
-    if (size + TCL_UTF_MAX >= esPtr->msize) {
+    /* We drop one third when are at least 2/3 full */
+    /* condition is (size >= max*2/3) <=> (size*3 >= max*2) */
+    if (numchars*3 >= esPtr->input.max*2) {
 	/*
 	 * In theory, interact could be invoked when this situation
 	 * already exists, hence the "probably" in the warning below
@@ -442,22 +462,25 @@ intRead(interp,esPtr,warnOnBufferFull,interruptible,key)
 	exp_buffer_shuffle(interp,esPtr,0,INTER_OUT,"interact");
     }
     if (!interruptible) {
-	cc = Tcl_ReadChars(esPtr->channel,
-		esPtr->buffer,
-		esPtr->msize - (size / TCL_UTF_MAX),
-		1 /* append */);
+        cc = Tcl_ReadChars(esPtr->channel, esPtr->input.newchars,
+			   esPtr->input.max - esPtr->input.use,
+			   0 /* no append */);
     } else {
 #ifdef SIMPLE_EVENT
-	cc = intIRead(esPtr->channel,
-		esPtr->buffer,
-		esPtr->msize - (size / TCL_UTF_MAX),
-		1 /* append */);
+        cc = intIRead(esPtr->channel, esPtr->input.newchars,
+		      esPtr->input.max - esPtr->input.use,
+		      0 /* no append */);
 #endif
     }
 
     if (cc > 0) {
+        memcpy (esPtr->input.buffer + esPtr->input.use,
+		Tcl_GetUnicodeFromObj (esPtr->input.newchars, NULL),
+		cc * sizeof (Tcl_UniChar));
+	esPtr->input.use += cc;
+
 	expDiagLog("spawn id %s sent <",esPtr->name);
-	expDiagLogU(expPrintify(eobOld));
+	expDiagLogU(expPrintifyUni(eobOld,cc));
 	expDiagLogU(">\r\n");
 
 	esPtr->key = key;
@@ -534,21 +557,23 @@ sigchld_handler()
  * process or the child (surrogate).
  */
 static int
-intIRead(channel,obj,size,flags);
-Tcl_Channel channel;
-Tcl_Obj *obj;
-int size;
-int flags;
+intIRead(
+    Tcl_Channel channel,
+    Tcl_Obj *obj,
+    int size,
+    int flags)
 {
     int cc = EXP_CHILD_EOF;
 
     if (deferred_interrupt) return(cc);
 
+    if (
 #ifdef HAVE_SIGLONGJMP
-    if (0 == sigsetjmp(env,1)) {
+	0 == sigsetjmp(env,1)
 #else
-    if (0 == setjmp(env)) {
+	0 == setjmp(env)
 #endif  /* HAVE_SIGLONGJMP */
+	) {
 	reading = TRUE;
 	cc = Tcl_ReadChars(channel,obj,size,flags);
     }
@@ -561,31 +586,30 @@ int flags;
 #define SPAWNED_PROCESS_DIED	-3
 
 static void
-clean_up_after_child(interp,esPtr)
-Tcl_Interp *interp;
-ExpState *esPtr;
+clean_up_after_child(
+    Tcl_Interp *interp,
+    ExpState *esPtr)
 {
     expWaitOnOne(); /* wait for slave */
     expWaitOnOne(); /* wait for child */
 
     deferred_interrupt = FALSE;
     if (esPtr->close_on_eof) {
-      exp_close(interp,esPtr);
-    }
+    exp_close(interp,esPtr);
+}
 }
 #endif /*SIMPLE_EVENT*/
 
 static int
-update_interact_fds(interp,esPtrCount,esPtrToInput,esPtrs,input_base,
-			do_indirect,config_count,real_tty_caller)
-Tcl_Interp *interp;
-int *esPtrCount;
-Tcl_HashTable **esPtrToInput;	/* map from ExpStates to "struct inputs" */
-ExpState ***esPtrs;
-struct input *input_base;
-int do_indirect;		/* if true do indirects */
-int *config_count;
-int *real_tty_caller;
+update_interact_fds(
+    Tcl_Interp *interp,
+    int *esPtrCount,
+    Tcl_HashTable **esPtrToInput,	/* map from ExpStates to "struct inputs" */
+    ExpState ***esPtrs,
+    struct input *input_base,
+    int do_indirect,		/* if true do indirects */
+    int *config_count,
+    int *real_tty_caller)
 {
 	struct input *inp;
 	struct output *outp;
@@ -669,12 +693,12 @@ int *real_tty_caller;
 
 /*ARGSUSED*/
 static char *
-inter_updateproc(clientData, interp, name1, name2, flags)
-ClientData clientData;
-Tcl_Interp *interp;	/* Interpreter containing variable. */
-char *name1;		/* Name of variable. */
-char *name2;		/* Second part of variable name. */
-int flags;		/* Information about what happened. */
+inter_updateproc(
+    ClientData clientData,
+    Tcl_Interp *interp,	/* Interpreter containing variable. */
+    char *name1,	/* Name of variable. */
+    char *name2,	/* Second part of variable name. */
+    int flags)		/* Information about what happened. */
 {
 	exp_configure_count++;
 	return 0;
@@ -687,16 +711,19 @@ static char interpreter_cmd[] = "interpreter";
 
 /*ARGSUSED*/
 int
-Exp_InteractObjCmd(clientData, interp, objc, objv)
-ClientData clientData;
-Tcl_Interp *interp;
-int objc;
-Tcl_Obj *CONST objv[];		/* Argument objects. */
+Exp_InteractObjCmd(
+    ClientData clientData,
+    Tcl_Interp *interp,
+    int objc,
+    Tcl_Obj *CONST initial_objv[])		/* Argument objects. */
 {
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
 
     Tcl_Obj *CONST *objv_copy;	/* original, for error messages */
+    Tcl_Obj **objv = (Tcl_Obj **) initial_objv;
     char *string;
+    Tcl_UniChar *ustring;
+
 #ifdef SIMPLE_EVENT
     int pid;
 #endif /*SIMPLE_EVENT*/
@@ -757,14 +784,28 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 
     int key;
     int configure_count;	/* monitor reconfigure events */
+    Tcl_Obj* new_cmd = NULL;
 
     if ((objc == 2) && exp_one_arg_braced(objv[1])) {
-	return(exp_eval_with_one_arg(clientData,interp,objv));
+	/* expect {...} */
+
+	new_cmd = exp_eval_with_one_arg(clientData,interp,objv);
+	if (!new_cmd) return TCL_ERROR;
+
+	/* Replace old arguments with result of reparse */
+	Tcl_ListObjGetElements (interp, new_cmd, &objc, &objv);
+
     } else if ((objc == 3) && streq(Tcl_GetString(objv[1]),"-brace")) {
+	/* expect -brace {...} ... fake command line for reparsing */
+
 	Tcl_Obj *new_objv[2];
 	new_objv[0] = objv[0];
 	new_objv[1] = objv[2];
-	return(exp_eval_with_one_arg(clientData,interp,new_objv));
+
+	new_cmd = exp_eval_with_one_arg(clientData,interp,new_objv);
+	if (!new_cmd) return TCL_ERROR;
+	/* Replace old arguments with result of reparse */
+	Tcl_ListObjGetElements (interp, new_cmd, &objc, &objv);
     }
 
     objv_copy = objv;
@@ -839,7 +880,7 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 
 	    if (Tcl_GetIndexFromObj(interp, *objv, switches, "switch", 0,
 		    &index) != TCL_OK) {
-		return TCL_ERROR;
+		goto error;
 	    }
 	    switch ((enum switches) index) {
 		case EXP_SWITCH_DASH:
@@ -850,7 +891,7 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 		case EXP_SWITCH_REGEXP:
 		    if (objc < 1) {
 			Tcl_WrongNumArgs(interp,1,objv_copy,"-re pattern");
-			return(TCL_ERROR);
+		    goto error;
 		    }
 		    next_re = TRUE;
 		    objc--;
@@ -864,7 +905,7 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 
 		    if (!(Tcl_GetRegExpFromObj(interp, *objv,
 			    TCL_REG_ADVANCED|TCL_REG_BOSONLY))) {
-			return TCL_ERROR;
+		    goto error;
 		    }
 		    goto pattern;
 		case EXP_SWITCH_INPUT:
@@ -888,11 +929,13 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 		    objc--;objv++;
 		    if (objc < 1) {
 			Tcl_WrongNumArgs(interp,1,objv_copy,"-input spawn_id");
-			return(TCL_ERROR);
+		    goto error;
 		    }
 		    inp->i_list = exp_new_i_complex(interp,Tcl_GetString(*objv),
 			    EXP_TEMPORARY,inter_updateproc);
-		    if (!inp->i_list) return TCL_ERROR;
+		if (!inp->i_list) {
+		    goto error;
+		}
 		    break;
 		case EXP_SWITCH_OUTPUT: {
 		    struct output *tmp;
@@ -910,11 +953,13 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 		    objc--;objv++;
 		    if (objc < 1) {
 			Tcl_WrongNumArgs(interp,1,objv_copy,"-output spawn_id");
-			return(TCL_ERROR);
+		    goto error;
 		    }
 		    outp->i_list = exp_new_i_complex(interp,Tcl_GetString(*objv),
 			    EXP_TEMPORARY,inter_updateproc);
-		    if (!outp->i_list) return TCL_ERROR;
+		if (!outp->i_list) {
+		    goto error;
+		}
 		    outp->action_eof = &action_eof;
 		    action_eof_ptr = &outp->action_eof;
 		    break;
@@ -923,7 +968,7 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 		    objc--;objv++;
 		    if (objc < 1) {
 			Tcl_WrongNumArgs(interp,1,objv_copy,"-u spawn_id");
-			return(TCL_ERROR);
+		    goto error;
 		    }
 		    replace_user_by_process = *objv;
 
@@ -937,8 +982,8 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 		    end_km = &input_default->keymap;
 
 		    if (dash_o_count > 0) {
-		      exp_error(interp,"cannot use -o more than once");
-		      return TCL_ERROR;
+			exp_error(interp,"cannot use -o more than once");
+			goto error;
 		    }
 		    dash_o_count++;
 
@@ -1006,11 +1051,11 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 		    objc--;objv++;
 		    if (objc < 1) {
 			Tcl_WrongNumArgs(interp,1,objv_copy,"-timeout time");
-			return(TCL_ERROR);
+			goto error;
 		    }
 
 		    if (Tcl_GetIntFromObj(interp, *objv, &t) != TCL_OK) {
-			return TCL_ERROR;
+		    goto error;
 		    }
 		    objc--;objv++;
 		    if (t != -1)
@@ -1086,10 +1131,10 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 		    objc--;objv++;
 		    if (objc < 1) {
 			Tcl_WrongNumArgs(interp,1,objv_copy,"timeout time [action]");
-			return(TCL_ERROR);
+		    goto error;
 		    }
 		    if (Tcl_GetIntFromObj(interp, *objv, &t) != TCL_OK) {
-			return TCL_ERROR;
+		    goto error;
 		    }
 		    objc--;objv++;
 
@@ -1183,13 +1228,15 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 	struct output *o = new(struct output);
 	if (!chanName) {
 	    if (!(esPtr = expStateCurrent(interp,1,1,0))) {
-		return(TCL_ERROR);
+		goto error;
 	    }
 	    o->i_list = exp_new_i_simple(esPtr,EXP_TEMPORARY);
 	} else {
 	    o->i_list = exp_new_i_complex(interp,Tcl_GetString(chanName),
 		    EXP_TEMPORARY,inter_updateproc);
-	    if (!o->i_list) return TCL_ERROR;
+	    if (!o->i_list) {
+		goto error;
+	    }
 	}
 	o->next = 0;	/* no one else */
 	o->action_eof = &action_eof;
@@ -1215,11 +1262,13 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 	input_user->i_list = exp_new_i_complex(interp,
 		Tcl_GetString(replace_user_by_process),
 		EXP_TEMPORARY,inter_updateproc);
-	if (!input_user->i_list) return TCL_ERROR;
+	if (!input_user->i_list) 
+	    goto error;
 	input_default->output->i_list = exp_new_i_complex(interp,
 		Tcl_GetString(replace_user_by_process),
 		EXP_TEMPORARY,inter_updateproc);
-	if (!input_default->output->i_list) return TCL_ERROR;
+	if (!input_default->output->i_list) 
+	    goto error;
     }
 
     /*
@@ -1235,7 +1284,7 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 	    && (input_default->i_list->state_list->esPtr == EXP_SPAWN_ID_BAD)) {
 	if (!chanName) {
 	    if (!(esPtr = expStateCurrent(interp,1,1,0))) {
-		return(TCL_ERROR);
+		goto error;
 	    }
 	    input_default->i_list->state_list->esPtr = esPtr;
 	} else {
@@ -1243,7 +1292,8 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 	    exp_free_i(interp,input_default->i_list,inter_updateproc);
 	    input_default->i_list = exp_new_i_complex(interp,Tcl_GetString(chanName),
 		    EXP_TEMPORARY,inter_updateproc);
-	    if (!input_default->i_list) return TCL_ERROR;
+	    if (!input_default->i_list)
+		goto error;
 	}
     }
 
@@ -1260,7 +1310,7 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
     if (input_user->i_list->state_list && input_default->i_list->state_list
 	    && (input_user->i_list->state_list->esPtr == input_default->i_list->state_list->esPtr)) {
 	exp_error(interp,"cannot interact with self - set spawn_id to a spawned process");
-	return(TCL_ERROR);
+	goto error;
     }
 
     esPtrs = 0;
@@ -1304,6 +1354,7 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 	int attempt_match = TRUE;
 	struct input *soonest_input;
 	int timeout;	/* current as opposed to default_timeout */
+	Tcl_Time temp_time;
 
 	/* calculate how long to wait */
 	/* by finding shortest remaining timeout */
@@ -1320,7 +1371,8 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 		}
 	    }
 
-	    time(&previous_time);
+	    Tcl_GetTime (&temp_time);
+	    previous_time = temp_time.sec;
 	    /* timestamp here rather than simply saving old */
 	    /* current time (after ready()) to account for */
 	    /* possibility of slow actions */
@@ -1339,7 +1391,8 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 	}
 
 	rc = exp_get_next_event(interp,esPtrs,input_count,&u,timeout,key);
-	if (rc == EXP_TCLERROR) return(TCL_ERROR);
+	if (rc == EXP_TCLERROR)
+	    goto error;
 	if (rc == EXP_RECONFIGURE) continue;
 	if (rc == EXP_TIMEOUT) {
 	    if (timeout_simple) {
@@ -1354,7 +1407,8 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 	if (!timeout_simple) {
 	    int time_diff;
 
-	    time(&current_time);
+	    Tcl_GetTime (&temp_time);
+	    current_time = temp_time.sec;
 	    time_diff = current_time - previous_time;
 
 	    /* update all timers */
@@ -1379,7 +1433,7 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 	switch (rc) {
 	    case EXP_DATA_NEW:
 		cc = intRead(interp,u,1,0,key);
-		if (cc > 0) break;
+		if (cc > 0 || Tcl_InputBlocked(u->channel)) break;
 
 		rc = EXP_EOF;
 		/*
@@ -1452,7 +1506,7 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 		     * Following should eventually be rewritten to ...WriteCharsAnd...
 		     */
 		    int wc = expWriteBytesAndLogIfTtyU(fdp->esPtr,
-			    Tcl_GetString(u->buffer) + u->printed,
+						       u->input.buffer + u->printed,
 			    print - u->printed);
 		    if (wc < 0) {
 			expDiagLog("interact: write on spawn id %s failed (%s)\r\n",fdp->esPtr->name,Tcl_PosixError(interp));
@@ -1503,17 +1557,17 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 	    skip += matchLen;
 	    size -= skip;
 	    if (size) {
-		string = Tcl_GetString(u->buffer);
-		memmove(string, string + skip, size);
+		ustring = u->input.buffer;
+		memmove(ustring, ustring + skip, size * sizeof(Tcl_UniChar));
 	    }
 	} else {
-	    string = Tcl_GetString(u->buffer);
+	    ustring = u->input.buffer;
 	    if (skip) {
 		size -= skip;
-		memcpy(string, string + skip, size);
+		memcpy(ustring, ustring + skip, size * sizeof(Tcl_UniChar));
 	    }
 	}
-	Tcl_SetObjLength(u->buffer,size);
+	u->input.use = size;
 
 	/* now update printed based on total amount skipped */
 
@@ -1535,7 +1589,7 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 	  if (u->close_on_eof) {
 	    exp_close(interp,u);
 	  }
-	  need_to_close_master = FALSE;
+	    need_to_close_master = FALSE;
 	}
 
 	if (action) {
@@ -1624,7 +1678,8 @@ got_action:
 					timeout = inp->timeout_remaining;
 			}
 
-			time(&previous_time);
+			Tcl_GetTime (&temp_time);
+			previous_time = temp_time.sec;
 			/* timestamp here rather than simply saving old */
 			/* current time (after ready()) to account for */
 			/* possibility of slow actions */
@@ -1639,7 +1694,8 @@ got_action:
 		if (!timeout_simple) {
 			int time_diff;
 
-			time(&current_time);
+			Tcl_GetTime (&temp_time);
+			current_time = temp_time.sec;
 			time_diff = current_time - previous_time;
 
 			/* update all timers */
@@ -1722,7 +1778,7 @@ got_action:
 			for (fdp = outp->i_list->state_list;fdp;fdp=fdp->next) {
 			    /* send to channel (and log if chan is stdout or devtty) */
 			    int wc = expWriteBytesAndLogIfTtyU(fdp->esPtr,
-				    Tcl_GetString(u->buffer) + u->printed,
+							       u->input.buffer + u->printed,
 				    print - u->printed);
 			    if (wc < 0) {
 				expDiagLog("interact: write on spawn id %s failed (%s)\r\n",fdp->esPtr->name,Tcl_PosixError(interp));
@@ -1852,7 +1908,8 @@ got_action:
 					timeout = inp->timeout_remaining;
 			}
 
-			time(&previous_time);
+			Tcl_GetTime (&temp_time);
+			previous_time = temp_time.sec;
 			/* timestamp here rather than simply saving old */
 			/* current time (after ready()) to account for */
 			/* possibility of slow actions */
@@ -1866,7 +1923,8 @@ got_action:
 		if (!timeout_simple) {
 			int time_diff;
 
-			time(&current_time);
+			Tcl_GetTime (&temp_time);
+			current_time = temp_time.sec;
 			time_diff = current_time - previous_time;
 
 			/* update all timers */
@@ -1961,7 +2019,7 @@ got_action:
 			for (fdp = outp->i_list->state_list;fdp;fdp=fdp->next) {
 			    /* send to channel (and log if chan is stdout or devtty) */
 			    int wc = expWriteBytesAndLogIfTtyU(fdp->esPtr,
-				    Tcl_GetString(u->buffer) + u->printed,
+							       u->input.buffer + u->printed,
 				    print - u->printed);
 			    if (wc < 0) {
 				expDiagLog("interact: write on spawn id %s failed (%s)\r\n",fdp->esPtr->name,Tcl_PosixError(interp));
@@ -2086,15 +2144,20 @@ got_action:
     free_input(interp,input_base);
     free_action(action_base);
 
+    if (new_cmd) { Tcl_DecrRefCount (new_cmd); }
     return(status);
+
+ error:
+    if (new_cmd) { Tcl_DecrRefCount (new_cmd); }
+    return TCL_ERROR;
 }
 
 /* version of Tcl_Eval for interact */ 
 static int
-inter_eval(interp,action,esPtr)
-Tcl_Interp *interp;
-struct action *action;
-ExpState *esPtr;
+inter_eval(
+    Tcl_Interp *interp,
+    struct action *action,
+    ExpState *esPtr)
 {
     int status;
 
@@ -2113,8 +2176,7 @@ ExpState *esPtr;
 }
 
 static void
-free_keymap(km)
-struct keymap *km;
+free_keymap(struct keymap *km)
 {
 	if (km == 0) return;
 	free_keymap(km->next);
@@ -2123,8 +2185,7 @@ struct keymap *km;
 }
 
 static void
-free_action(a)
-struct action *a;
+free_action(struct action *a)
 {
 	struct action *next;
 
@@ -2136,9 +2197,9 @@ struct action *a;
 }
 
 static void
-free_input(interp,i)
-Tcl_Interp *interp;
-struct input *i;
+free_input(
+    Tcl_Interp *interp,
+    struct input *i)
 {
 	if (i == 0) return;
 	free_input(interp,i->next);
@@ -2150,8 +2211,7 @@ struct input *i;
 }
 
 static struct action *
-new_action(base)
-struct action **base;
+new_action(struct action **base)
 {
 	struct action *o = new(struct action);
 
@@ -2163,9 +2223,9 @@ struct action **base;
 }
 
 static void
-free_output(interp,o)
-Tcl_Interp *interp;
-struct output *o;
+free_output(
+    Tcl_Interp *interp,
+    struct output *o)
 {
 	if (o == 0) return;
 	free_output(interp,o->next);
@@ -2180,8 +2240,7 @@ static struct exp_cmd_data cmd_data[]  = {
 {0}};
 
 void
-exp_init_interact_cmds(interp)
-Tcl_Interp *interp;
+exp_init_interact_cmds(Tcl_Interp *interp)
 {
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
 
@@ -2194,3 +2253,11 @@ Tcl_Interp *interp;
     Tcl_IncrRefCount(tsdPtr->cmdObjInterpreter);
 #endif
 }
+
+/*
+ * Local Variables:
+ * mode: c
+ * c-basic-offset: 4
+ * fill-column: 78
+ * End:
+ */
