@@ -69,6 +69,9 @@ with openpty which supports 4000 while ptmx supports 60. */
 #else
 #include <stdlib.h>
 #endif
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
 
 #ifdef HAVE_SYSMACROS_H
 #include <sys/sysmacros.h>
@@ -627,10 +630,10 @@ int control;	/* if 1, enable pty trapping of close/open/ioctl */
 }
 
 int
-exp_getptyslave(ttycopy,ttyinit,stty_args)
-int ttycopy;
-int ttyinit;
-char *stty_args;
+exp_getptyslave(
+    int ttycopy,
+    int ttyinit,
+    CONST char *stty_args)
 {
 	int slave, slave2;
 	char buf[10240];
