@@ -5,7 +5,9 @@
 # in this directory.
 
 package require tcltest
-namespace import ::tcltest::*
+# do this in a way that is backward compatible for Tcl 8.3
+namespace import ::tcltest::test ::tcltest::cleanupTests
+package require Expect
 
 set ::tcltest::testSingleFile false
 set ::tcltest::testsDirectory [file dirname [info script]]
